// IOCallbacks.cpp
#include "IOCallbacks.h"
#include "../FileSystem/FileSystem.h"
#include "../Logging/Logger.h"

using namespace Convex;

#define FFMPEG_BUFFER_SIZE 1024 * 256

extern "C"
{
#include <libavcodec/avcodec.h>

int FFMpegReadFunc(void *opaque, uint8_t *buf, int buf_size)
{
	File *file = (File*) opaque;
	if (!file) return -1;
	return file->Read(buf, buf_size);
}

int64_t FFMpegSeekFunc(void *opaque, int64_t offset, int whence)
{
	File *file = (File*) opaque;
	if (!file) return -1;

	if (whence == AVSEEK_SIZE) return file->Size();

	SeekMode mode;
	switch (whence) {
		case SEEK_SET: mode = Seek_Set;
			break;
		case SEEK_CUR: mode = Seek_Cur;
			break;
		case SEEK_END: mode = Seek_End;
			break;
		default:
			return -1;
			break;
	}

	return file->Seek(offset, mode);
}

} // end extern c

void FFMpegIO::CleanUp()
{
	if (buffer) delete [] buffer;
	buffer = NULL;
	if (file) ConvexFS->CloseFile(file);
	file = NULL;
}

FFMpegIO::FFMpegIO(const UniString &filename)
{
	file = NULL;
	buffer = NULL;

	try {
		file = ConvexFS->OpenFile(filename);
		if (!file) throw("Failed to open file");

		buffer = new unsigned char[FFMPEG_BUFFER_SIZE + FF_INPUT_BUFFER_PADDING_SIZE];
		if (!buffer) throw("Failed to allocate a buffer for video i/o");

		init_put_byte(&byte_io_context, buffer, FFMPEG_BUFFER_SIZE, 0 /*write flag*/,
			file, FFMpegReadFunc, NULL, FFMpegSeekFunc);

	} catch (const char *message) {
		ConvexError(filename + ": " + message);
		CleanUp();
	}
}

FFMpegIO::~FFMpegIO()
{
	CleanUp();
}

ByteIOContext *FFMpegIO::GetIOContext()
{
	if (!buffer || !file) return NULL;
	return &byte_io_context;
}

Integer64 FFMpegIO::Tell() const {
	if (!file) return 0;
	return file->Tell();
}
