// VideoOps.cpp

#include "VideoOps.h"
#include "../Graphics/GraphicsContext.h"
#include "../FileSystem/FileSystem.h"

using namespace Convex;

Video *VideoOps::CreateVideo(const UniString &filename)
{
	Video *ret = new Video(filename);
	if (ret->pFormatCtx == NULL ||
		ret->pCodecCtx == NULL ||
		ret->videoStream == -1 ||
		ret->pCodec == NULL ||
		ret->pFrame == NULL ||
		ret->pFrameRGB == NULL ||
		ret->buffer == NULL)
	{
		delete ret;
		return NULL;
	}
	return ret;
}

void VideoOps::DeleteVideo(Video *vid)
{
	if (vid) delete vid;
}

VideoTexture *VideoOps::ActualCreateVideoTexture(GraphicsContext *context, const UniString &filename,
							bool convert_rgb)
{
	VideoTexture *ret = new VideoTexture(context, filename, convert_rgb);
	if (!ret->tex || !ret->video_loaded) {
		delete ret;
		return NULL;
	}

	return ret;
}

VideoTexture *VideoOps::CreateVideoTexture(GraphicsContext *context, const UniString &filename, 
			bool convert_rgb, const UniStringSet *path)
{
	// find the file in the path
	UniString full_filename;
	bool found = ConvexFS->FileExists(filename, path, &full_filename);
	if (found) {
		return ActualCreateVideoTexture(context, full_filename, convert_rgb);
	}
	return NULL;
}


void VideoOps::DeleteVideoTexture(VideoTexture *vid)
{
	if (vid) delete vid;
}