// VideoTexture.cpp

#include "VideoTexture.h"
#include "../Graphics/GraphicsContext.h"
#include "../Graphics/GraphicsUtilities.h"
#include "../Graphics/DynamicTexture.h"

using namespace Convex;

VideoTexture::VideoTexture(GraphicsContext *context, 
		const UniString &filename, bool convert_rgb) : Video(filename, convert_rgb)
{
	gc = NULL;
	tex = NULL;

	if (pFormatCtx == NULL ||
		pCodecCtx == NULL ||
		videoStream == -1 ||
		pCodec == NULL ||
		pFrame == NULL ||
		pFrameRGB == NULL ||
		buffer == NULL) return;

	if (!context) return;
	gc = context;

	tex = gc->CreateDynamicTexture(Video::GetWidth(), Video::GetHeight());
}

VideoTexture::~VideoTexture()
{
	if (tex) gc->DestroyTexture(tex);
}

void VideoTexture::UpdatePixels()
{
	if (tex) tex->UpdatePixels(Video::GetBuffer(), 
		Video::GetWidth(), Video::GetHeight(), Video::GetStride());
}

DynamicTexture *VideoTexture::GetTexture()
{
	return tex;
}