// VideoTextureManager.h

#ifndef _VIDEO_TEXTURE_MANAGER_H_
#define _VIDEO_TEXTURE_MANAGER_H_

#include "../Graphics/TextureManager.h"
#include "../Utility/Clock.h"

namespace Convex
{
	class VideoTexture;

	class VideoTextureManager : public TextureManager
	{
	protected:
		struct RefVideoTexture
		{
			VideoTexture *tex;
			int ref_count;
			Clock clock;

			RefVideoTexture() {tex = NULL; ref_count = 0;}
		};
		map<UniString, RefVideoTexture> video_textures;
		
		Texture *CreateVideoTexture(const UniString &filename);
		void DeleteVideoTexture(Texture *tex);
		UniStringList CollectVideoTextures() const;
	public:
		VideoTextureManager(GraphicsContext *context);
		virtual ~VideoTextureManager();

		virtual Texture *CreateTexture(const UniString &filename, int quality);
		virtual void DeleteTexture(Texture *tex);

	#ifdef DEVELOPMENT_VERSION
		/*!
			/brief Creates a list of absolute filenames of all managed textures
			Available only in DEVELOPMENT version
		*/
		virtual UniStringList CollectTextures() const;
	#endif

		void UpdateVideoTextures();
	};

} // end namespace Convex

#endif // ndef _VIDEO_TEXTURE_MANAGER_H_
